@@ -1,9 +1,11 @@
-Sequel.require %w'emulate_offset_with_row_number split_alter_table', 'adapters/utils/'
+Sequel.require %w'emulate_offset_with_row_number split_alter_table', 'adapters/utils'
 
 module Sequel
   Dataset::NON_SQL_OPTIONS << :disable_insert_output
   module MSSQL
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
+
       AUTO_INCREMENT = 'IDENTITY(1,1)'.freeze
       SERVER_VERSION_RE = /^(\d+)\.(\d+)\.(\d+)/.freeze
       SERVER_VERSION_SQL = "SELECT CAST(SERVERPROPERTY('ProductVersion') AS varchar)".freeze
@@ -21,12 +23,62 @@ module Sequel
       # strings.  True by default for compatibility, can be set to false for a possible
       # performance increase.  This sets the default for all datasets created from this
       # Database object.
-      attr_accessor :mssql_unicode_strings
+      attr_reader :mssql_unicode_strings
+
+      def mssql_unicode_strings=(v)
+        @mssql_unicode_strings = v
+        reset_default_dataset
+      end
 
       # The types to check for 0 scale to transform :decimal types
       # to :integer.
       DECIMAL_TYPE_RE = /number|numeric|decimal/io
 
+      # Execute the given stored procedure with the given name.
+      #
+      # Options:
+      # :args :: Array of arguments to stored procedure.  Output parameters to
+      #          the function are specified using :output.  You can also name
+      #          output parameters and provide a type by using an array containing
+      #          :output, the type name, and the parameter name.
+      # :server :: The server/shard on which to execute the procedure.
+      #
+      # Examples:
+      #
+      #     DB.call_mssql_sproc(:SequelTest, {:args => ['input arg', :output]})
+      #     DB.call_mssql_sproc(:SequelTest, {:args => ['input arg', [:output, 'int', 'varname']]})
+      def call_mssql_sproc(name, opts=OPTS)
+        args = opts[:args] || []
+        names = ['@RC AS RESULT', '@@ROWCOUNT AS NUMROWS']
+        declarations = ['@RC int']
+        values = []
+
+        args.each_with_index do |v, i|
+          if v.is_a?(Array)
+            v, type, select = v
+          else
+            type = select = nil
+          end
+
+          if v == :output
+            type = "nvarchar(max)" unless type
+            varname = "var#{i}" unless varname
+            select ||= varname
+            names << "@#{varname} AS #{quote_identifier(select)}"
+            declarations << "@#{varname} #{type}"
+            values << "@#{varname} OUTPUT"
+          else
+            values << literal(v)
+          end
+        end
+
+        sql = "DECLARE #{declarations.join(', ')}; EXECUTE @RC = #{name} #{values.join(', ')}; SELECT #{names.join(', ')}"
+
+        ds = dataset.with_sql(sql)
+        ds = ds.server(opts[:server]) if opts[:server]
+        ds.first
+      end
+
       # Microsoft SQL Server uses the :mssql type.
       def database_type
         :mssql
@@ -39,7 +91,7 @@ module Sequel
 
       # Return foreign key information using the system views, including
       # :name, :on_delete, and :on_update entries in the hashes.
-      def foreign_key_list(table, opts={})
+      def foreign_key_list(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         schema, table = schema_and_table(table)
@@ -68,7 +120,7 @@ module Sequel
             referenced_schema = m.call(row[:schema])
             referenced_table = m.call(row[:table])
             h[row[:name]] = { :name      => m.call(row[:name]), 
-                              :table     => (referenced_schema == current_schema) ? referenced_table : :"#{referenced_schema}__#{referenced_table}",
+                              :table     => (referenced_schema == current_schema) ? referenced_table : Sequel.qualify(referenced_schema, referenced_table),
                               :columns   => [m.call(row[:column])], 
                               :key       => [m.call(row[:referenced_column])], 
                               :on_update => fk_action_map[row[:update_referential_action]], 
@@ -79,19 +131,24 @@ module Sequel
       end
 
       # Use the system tables to get index information
-      def indexes(table, opts={})
+      def indexes(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         indexes = {}
-        metadata_dataset.from(:sys__tables___t).
+        ds = metadata_dataset.from(:sys__tables___t).
          join(:sys__indexes___i, :object_id=>:object_id).
          join(:sys__index_columns___ic, :object_id=>:object_id, :index_id=>:index_id).
          join(:sys__columns___c, :object_id=>:object_id, :column_id=>:column_id).
          select(:i__name, :i__is_unique, :c__name___column).
          where{{t__name=>im.call(table)}}.
          where(:i__is_primary_key=>0, :i__is_disabled=>0).
-         order(:i__name, :ic__index_column_id).
-         each do |r|
+         order(:i__name, :ic__index_column_id)
+
+        if supports_partial_indexes?
+          ds = ds.where(:i__has_filter=>0)
+        end
+
+        ds.each do |r|
           index = indexes[m.call(r[:name])] ||= {:columns=>[], :unique=>(r[:is_unique] && r[:is_unique]!=0)}
           index[:columns] << m.call(r[:column])
         end
@@ -112,6 +169,11 @@ module Sequel
         @server_version
       end
         
+      # MSSQL 2008+ supports partial indexes.
+      def supports_partial_indexes?
+        dataset.send(:is_2008_or_later?)
+      end
+
       # MSSQL supports savepoints, though it doesn't support committing/releasing them savepoint
       def supports_savepoints?
         true
@@ -129,13 +191,13 @@ module Sequel
 
       # Microsoft SQL Server supports using the INFORMATION_SCHEMA to get
       # information on tables.
-      def tables(opts={})
+      def tables(opts=OPTS)
         information_schema_tables('BASE TABLE', opts)
       end
 
       # Microsoft SQL Server supports using the INFORMATION_SCHEMA to get
       # information on views.
-      def views(opts={})
+      def views(opts=OPTS)
         information_schema_tables('VIEW', opts)
       end
       
@@ -213,7 +275,7 @@ module Sequel
 
       # Commit the active transaction on the connection, does not commit/release
       # savepoints.
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         log_connection_execute(conn, commit_transaction_sql) unless _trans(conn)[:savepoint_level] > 1
       end
 
@@ -238,6 +300,17 @@ module Sequel
         run(ds.into(name).sql)
       end
     
+      DATABASE_ERROR_REGEXPS = {
+        /Violation of UNIQUE KEY constraint/ => UniqueConstraintViolation,
+        /conflicted with the (FOREIGN KEY.*|REFERENCE) constraint/ => ForeignKeyConstraintViolation,
+        /conflicted with the CHECK constraint/ => CheckConstraintViolation,
+        /column does not allow nulls/ => NotNullConstraintViolation,
+        /was deadlocked on lock resources with another process and has been chosen as the deadlock victim/ => SerializationFailure,
+      }.freeze
+      def database_error_regexps
+        DATABASE_ERROR_REGEXPS
+      end
+
       # The name of the constraint for setting the default value on the table and column.
       # The SQL used to select default constraints utilizes MSSQL catalog views which were introduced in 2005.
       # This method intentionally does not support MSSQL 2000.
@@ -258,6 +331,7 @@ module Sequel
       # support for clustered index type
       def index_definition_sql(table_name, index)
         index_name = index[:name] || default_index_name(table_name, index[:columns])
+        raise Error, "Partial indexes are not supported for this database" if index[:where] && !supports_partial_indexes?
         if index[:type] == :full_text
           "CREATE FULLTEXT INDEX ON #{quote_schema_table(table_name)} #{literal(index[:columns])} KEY INDEX #{literal(index[:key_index])}"
         else
@@ -270,7 +344,7 @@ module Sequel
         m = output_identifier_meth
         metadata_dataset.from(:information_schema__tables___t).
           select(:table_name).
-          filter(:table_type=>type, :table_schema=>(opts[:schema]||default_schema||'dbo').to_s).
+          filter(:table_type=>type, :table_schema=>(opts[:schema]||'dbo').to_s).
           map{|x| m.call(x[:table_name])}
       end
 
@@ -299,8 +373,12 @@ module Sequel
       # The closest MSSQL equivalent of a boolean datatype is the bit type.
       def schema_column_type(db_type)
         case db_type
-        when /\A(bit)\z/io
+        when /\A(?:bit)\z/io
           :boolean
+        when /\A(?:(?:small)?money)\z/io
+          :decimal
+        when /\A(timestamp|rowversion)\z/io
+          :blob
         else
           super
         end
@@ -330,7 +408,7 @@ module Sequel
               :table_schema => :table_schema, :table_name => :table_name).
          select(:column_name___column, :data_type___db_type, :character_maximum_length___max_chars, :column_default___default, :is_nullable___allow_null, :numeric_precision___column_size, :numeric_scale___scale).
          filter(:c__table_name=>tn)
-        if schema = opts[:schema] || default_schema
+        if schema = opts[:schema]
           ds.filter!(:c__table_schema=>schema)
         end
         ds.map do |row|
@@ -401,6 +479,7 @@ module Sequel
       APOS_RE = Dataset::APOS_RE
       DOUBLE_APOS = Dataset::DOUBLE_APOS
       INTO = Dataset::INTO
+      DOUBLE_BRACKET_CLOSE = ']]'.freeze
       DATEPART_SECOND_OPEN = "CAST((datepart(".freeze
       DATEPART_SECOND_MIDDLE = ') + datepart(ns, '.freeze
       DATEPART_SECOND_CLOSE = ")/1000000000.0) AS double precision)".freeze
@@ -421,14 +500,21 @@ module Sequel
       CASE_INSENSITIVE_COLLATION = 'Latin1_General_CI_AS'.freeze
       DEFAULT_TIMESTAMP_FORMAT = "'%Y-%m-%dT%H:%M:%S%N%z'".freeze
       FORMAT_DATE = "'%Y%m%d'".freeze
+      CROSS_APPLY = 'CROSS APPLY'.freeze
+      OUTER_APPLY = 'OUTER APPLY'.freeze
+      OFFSET = " OFFSET ".freeze
+      ROWS = " ROWS".freeze
+      ROWS_ONLY = " ROWS ONLY".freeze
+      FETCH_NEXT = " FETCH NEXT ".freeze
+
+      Sequel::Dataset.def_mutation_method(:disable_insert_output, :output, :module=>self)
 
       # Allow overriding of the mssql_unicode_strings option at the dataset level.
-      attr_accessor :mssql_unicode_strings
+      attr_writer :mssql_unicode_strings
 
-      # Copy the mssql_unicode_strings option from the +db+ object.
-      def initialize(db, opts={})
-        super
-        @mssql_unicode_strings = db.mssql_unicode_strings
+      # Use the database's mssql_unicode_strings setting if the dataset hasn't overridden it.
+      def mssql_unicode_strings
+        defined?(@mssql_unicode_strings) ? @mssql_unicode_strings : (@mssql_unicode_strings = db.mssql_unicode_strings)
       end
 
       # MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
@@ -469,16 +555,21 @@ module Sequel
         end
       end
       
+      # Uses CROSS APPLY to join the given table into the current dataset.
+      def cross_apply(table)
+        join_table(:cross_apply, table)
+      end
+
       # Disable the use of INSERT OUTPUT
       def disable_insert_output
         clone(:disable_insert_output=>true)
       end
 
-      # Disable the use of INSERT OUTPUT, modifying the receiver
-      def disable_insert_output!
-        mutation_method(:disable_insert_output)
+      # MSSQL treats [] as a metacharacter in LIKE expresions.
+      def escape_like(string)
+        string.gsub(/[\\%_\[\]]/){|m| "\\#{m}"}
       end
-
+   
       # There is no function on Microsoft SQL Server that does character length
       # and respects trailing spaces (datalength respects trailing spaces, but
       # counts bytes instead of characters).  Use a hack to work around the
@@ -495,7 +586,7 @@ module Sequel
       end
       
       # MSSQL uses the CONTAINS keyword for full text search
-      def full_text_search(cols, terms, opts = {})
+      def full_text_search(cols, terms, opts = OPTS)
         terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array)
         filter("CONTAINS (?, ?)", cols, terms)
       end
@@ -530,6 +621,11 @@ module Sequel
         lock_style(:dirty)
       end
 
+      # Uses OUTER APPLY to join the given table into the current dataset.
+      def outer_apply(table)
+        join_table(:outer_apply, table)
+      end
+
       # Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
       #
       # The first argument is the table to output into, and the second argument
@@ -555,17 +651,23 @@ module Sequel
         clone({:output => output})
       end
 
-      # An output method that modifies the receiver.
-      def output!(into, values)
-        mutation_method(:output, into, values)
-      end
-
-      # MSSQL uses [] to quote identifiers.  MSSQL does not support
-      # escaping of ], so you cannot use that character in an identifier.
+      # MSSQL uses [] to quote identifiers.
       def quoted_identifier_append(sql, name)
-        sql << BRACKET_OPEN << name.to_s << BRACKET_CLOSE
+        sql << BRACKET_OPEN << name.to_s.gsub(/\]/, DOUBLE_BRACKET_CLOSE) << BRACKET_CLOSE
       end
       
+      # On MSSQL 2012+ add a default order to the current dataset if an offset is used.
+      # The default offset emulation using a subquery would be used in the unordered
+      # case by default, and that also adds a default order, so it's better to just
+      # avoid the subquery.
+      def select_sql
+        if @opts[:offset] && !@opts[:order] && is_2012_or_later?
+          order(1).select_sql
+        else
+          super
+        end
+      end
+
       # The version of the database server.
       def server_version
         db.server_version(@opts[:server])
@@ -632,7 +734,7 @@ module Sequel
       # dataset.  If OUTPUT is already set, use existing returning values.  If OUTPUT
       # is only set to return a single columns, return an array of just that column.
       # Otherwise, return an array of hashes.
-      def _import(columns, values, opts={})
+      def _import(columns, values, opts=OPTS)
         if opts[:return] == :primary_key && !@opts[:output]
           output(nil, [SQL::QualifiedIdentifier.new(:inserted, first_primary_key)])._import(columns, values, opts)
         elsif @opts[:output]
@@ -662,6 +764,11 @@ module Sequel
         server_version >= 10000000
       end
 
+      # Whether we are using SQL Server 2012 or later.
+      def is_2012_or_later?
+        server_version >= 11000000
+      end
+
       # Use strict ISO-8601 format with T between date and time,
       # since that is the format that is multilanguage and not
       # DATEFORMAT dependent.
@@ -689,6 +796,11 @@ module Sequel
         end
       end
       alias update_from_sql delete_from2_sql
+
+      # Microsoft SQL Server 2012 has native support for offsets, but only for ordered datasets.
+      def emulate_offset_with_row_number?
+        super && !(is_2012_or_later? && @opts[:order])
+      end
       
       # Return the first primary key for the current table.  If this table has
       # multiple primary keys, this will only return one of them.  Used by #_import.
@@ -719,6 +831,18 @@ module Sequel
         end
       end
 
+      # Handle CROSS APPLY and OUTER APPLY JOIN types
+      def join_type_sql(join_type)
+        case join_type
+        when :cross_apply
+          CROSS_APPLY
+        when :outer_apply
+          OUTER_APPLY
+        else
+          super
+        end
+      end
+
       # MSSQL uses a literal hexidecimal number for blob strings
       def literal_blob_append(sql, v)
         sql << HEX_START << v.unpack(HSTAR).first
@@ -747,7 +871,8 @@ module Sequel
         BOOL_TRUE
       end
       
-      # MSSQL adds the limit before the columns
+      # MSSQL adds the limit before the columns, except on 2012+ when using
+      # offsets on ordered queries.
       def select_clause_methods
         SELECT_CLAUSE_METHODS
       end
@@ -763,6 +888,8 @@ module Sequel
       # to allow the limit to be a bound variable.
       def select_limit_sql(sql)
         if l = @opts[:limit]
+          return if is_2012_or_later? && @opts[:order] && @opts[:offset]
+
           if is_2005_or_later?
             sql << TOP_PAREN
             literal_append(sql, l)
@@ -787,6 +914,25 @@ module Sequel
         end
       end
 
+      # On 2012+ when there is an order with an offset, append the offset (and possible
+      # limit) at the end of the order clause.
+      def select_order_sql(sql)
+        super
+        if is_2012_or_later? && @opts[:order]
+          if o = @opts[:offset]
+            sql << OFFSET
+            literal_append(sql, o)
+            sql << ROWS
+
+            if l = @opts[:limit]
+              sql << FETCH_NEXT
+              literal_append(sql, l)
+              sql << ROWS_ONLY
+            end
+          end
+        end
+      end
+
       # SQL fragment for MSSQL's OUTPUT clause.
       def output_sql(sql)
         return unless supports_output_clause?
