@@ -1,29 +1,21 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
+SEQUEL_ADAPTER_TEST = :sqlite
 
-unless defined?(SQLITE_DB)
-  unless defined? SQLITE_URL
-    if defined?(RUBY_ENGINE) && RUBY_ENGINE == 'jruby'
-      SQLITE_URL = 'jdbc:sqlite::memory:'
-    else
-      SQLITE_URL = 'sqlite:/'
-    end
-  end
-  SQLITE_DB = Sequel.connect(ENV['SEQUEL_SQLITE_SPEC_DB']||SQLITE_URL)
-end
-INTEGRATION_DB = SQLITE_DB unless defined?(INTEGRATION_DB)
+require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 describe "An SQLite database" do
   before do
-    @db = SQLITE_DB
+    @db = DB
     @fk = @db.foreign_keys
   end
   after do
+    @db.drop_table?(:fk)
     @db.foreign_keys = @fk
     @db.case_sensitive_like = true
+    @db.use_timestamp_timezones = false
     Sequel.datetime_class = Time
   end
 
-  if SQLITE_DB.auto_vacuum == :none
+  if DB.auto_vacuum == :none
     specify "should support getting pragma values" do
       @db.pragma_get(:auto_vacuum).to_s.should == '0'
     end
@@ -56,12 +48,12 @@ describe "An SQLite database" do
   end
   
   specify "should support casting to Date by using the date function" do
-    @db.get('2012-10-20 11:12:13'.cast(Date)).should == '2012-10-20'
+    @db.get(Sequel.cast('2012-10-20 11:12:13', Date)).should == '2012-10-20'
   end
   
   specify "should support casting to Time or DateTime by using the datetime function" do
-    @db.get('2012-10-20'.cast(Time)).should == '2012-10-20 00:00:00'
-    @db.get('2012-10-20'.cast(DateTime)).should == '2012-10-20 00:00:00'
+    @db.get(Sequel.cast('2012-10-20', Time)).should == '2012-10-20 00:00:00'
+    @db.get(Sequel.cast('2012-10-20', DateTime)).should == '2012-10-20 00:00:00'
   end
   
   specify "should provide the SQLite version as an integer" do
@@ -74,16 +66,14 @@ describe "An SQLite database" do
     @db.foreign_keys = false
   end
   
-  if SQLITE_DB.sqlite_version >= 30619
-    specify "should enforce foreign key integrity if foreign_keys pragma is set" do
-      @db.foreign_keys = true
-      @db.create_table!(:fk){primary_key :id; foreign_key :parent_id, :fk}
-      @db[:fk].insert(1, nil)
-      @db[:fk].insert(2, 1)
-      @db[:fk].insert(3, 3)
-      proc{@db[:fk].insert(4, 5)}.should raise_error(Sequel::Error)
-    end
-  end
+  specify "should enforce foreign key integrity if foreign_keys pragma is set" do
+    @db.foreign_keys = true
+    @db.create_table!(:fk){primary_key :id; foreign_key :parent_id, :fk}
+    @db[:fk].insert(1, nil)
+    @db[:fk].insert(2, 1)
+    @db[:fk].insert(3, 3)
+    proc{@db[:fk].insert(4, 5)}.should raise_error(Sequel::Error)
+  end if DB.sqlite_version >= 30619
   
   specify "should not enforce foreign key integrity if foreign_keys pragma is unset" do
     @db.foreign_keys = false
@@ -93,24 +83,22 @@ describe "An SQLite database" do
   end
   
   specify "should support a use_timestamp_timezones setting" do
-    @db.create_table!(:time){Time :time}
-    @db[:time].insert(Time.now)
-    @db[:time].get(:time.cast_string).should =~ /[-+]\d\d\d\d\z/
-    @db.use_timestamp_timezones = false
-    @db[:time].delete
-    @db[:time].insert(Time.now)
-    @db[:time].get(:time.cast_string).should_not =~ /[-+]\d\d\d\d\z/
     @db.use_timestamp_timezones = true
+    @db.create_table!(:fk){Time :time}
+    @db[:fk].insert(Time.now)
+    @db[:fk].get(Sequel.cast(:time, String)).should =~ /[-+]\d\d\d\d\z/
+    @db.use_timestamp_timezones = false
+    @db[:fk].delete
+    @db[:fk].insert(Time.now)
+    @db[:fk].get(Sequel.cast(:time, String)).should_not =~ /[-+]\d\d\d\d\z/
   end
   
   specify "should provide a list of existing tables" do
-    @db.drop_table?(:testing)
+    @db.drop_table?(:fk)
     @db.tables.should be_a_kind_of(Array)
-    @db.tables.should_not include(:testing)
-    @db.create_table! :testing do
-      text :name
-    end
-    @db.tables.should include(:testing)
+    @db.tables.should_not include(:fk)
+    @db.create_table!(:fk){String :name}
+    @db.tables.should include(:fk)
   end
 
   specify "should support getting and setting the synchronous pragma" do
@@ -136,23 +124,24 @@ describe "An SQLite database" do
   end
   
   cspecify "should support timestamps and datetimes and respect datetime_class", :do, :jdbc, :amalgalite, :swift do
-    @db.create_table!(:time){timestamp :t; datetime :d}
+    @db.create_table!(:fk){timestamp :t; datetime :d}
+    @db.use_timestamp_timezones = true
     t1 = Time.at(1)
-    @db[:time] << {:t => t1, :d => t1}
-    @db[:time].map(:t).should == [t1]
-    @db[:time].map(:d).should == [t1]
+    @db[:fk] << {:t => t1, :d => t1}
+    @db[:fk].map(:t).should == [t1]
+    @db[:fk].map(:d).should == [t1]
     Sequel.datetime_class = DateTime
     t2 = Sequel.string_to_datetime(t1.iso8601)
-    @db[:time].map(:t).should == [t2]
-    @db[:time].map(:d).should == [t2]
+    @db[:fk].map(:t).should == [t2]
+    @db[:fk].map(:d).should == [t2]
   end
   
   specify "should support sequential primary keys" do
-    @db.create_table!(:with_pk) {primary_key :id; text :name}
-    @db[:with_pk] << {:name => 'abc'}
-    @db[:with_pk] << {:name => 'def'}
-    @db[:with_pk] << {:name => 'ghi'}
-    @db[:with_pk].order(:name).all.should == [
+    @db.create_table!(:fk) {primary_key :id; text :name}
+    @db[:fk] << {:name => 'abc'}
+    @db[:fk] << {:name => 'def'}
+    @db[:fk] << {:name => 'ghi'}
+    @db[:fk].order(:name).all.should == [
       {:id => 1, :name => 'abc'},
       {:id => 2, :name => 'def'},
       {:id => 3, :name => 'ghi'}
@@ -160,14 +149,47 @@ describe "An SQLite database" do
   end
   
   specify "should correctly parse the schema" do
-    @db.create_table!(:time2) {timestamp :t}
-    @db.schema(:time2, :reload=>true).should == [[:t, {:type=>:datetime, :allow_null=>true, :default=>nil, :ruby_default=>nil, :db_type=>"timestamp", :primary_key=>false}]]
+    @db.create_table!(:fk) {timestamp :t}
+    @db.schema(:fk, :reload=>true).should == [[:t, {:type=>:datetime, :allow_null=>true, :default=>nil, :ruby_default=>nil, :db_type=>"timestamp", :primary_key=>false}]]
+  end
+
+  specify "should handle and return BigDecimal values for numeric columns" do
+    DB.create_table!(:fk){numeric :d}
+    d = DB[:fk]
+    d.insert(:d=>BigDecimal.new('80.0'))
+    d.insert(:d=>BigDecimal.new('NaN'))
+    d.insert(:d=>BigDecimal.new('Infinity'))
+    d.insert(:d=>BigDecimal.new('-Infinity'))
+    ds = d.all
+    ds.shift.should == {:d=>BigDecimal.new('80.0')}
+    ds.map{|x| x[:d].to_s}.should == %w'NaN Infinity -Infinity'
+    DB
   end
 end
 
+describe "SQLite temporary views" do
+  before do
+    @db = DB
+    @db.drop_view(:items) rescue nil
+    @db.create_table!(:items){Integer :number}
+    @db[:items].insert(10)
+    @db[:items].insert(20)
+  end
+  after do
+    @db.drop_table?(:items)
+  end
+
+  specify "should be supported" do
+    @db.create_view(:items_view, @db[:items].where(:number=>10),  :temp=>true)
+    @db[:items_view].map(:number).should == [10]
+    @db.disconnect
+    lambda{@db[:items_view].map(:number)}.should raise_error(Sequel::DatabaseError)
+  end
+end
+    
 describe "SQLite type conversion" do
   before do
-    @db = SQLITE_DB
+    @db = DB
     @integer_booleans = @db.integer_booleans
     @db.integer_booleans = true
     @ds = @db[:items]
@@ -183,10 +205,10 @@ describe "SQLite type conversion" do
     @db.create_table(:items){TrueClass :a}
     @db[:items].insert(false)
     @db[:items].select_map(:a).should == [false]
-    @db[:items].select_map(:a+:a).should == [0]
+    @db[:items].select_map(Sequel.expr(:a)+:a).should == [0]
     @db[:items].update(:a=>true)
     @db[:items].select_map(:a).should == [true]
-    @db[:items].select_map(:a+:a).should == [2]
+    @db[:items].select_map(Sequel.expr(:a)+:a).should == [2]
   end
   
   specify "should handle integers/floats/strings/decimals in numeric/decimal columns" do
@@ -250,69 +272,55 @@ describe "SQLite type conversion" do
     @db[:items].update(:a=>'1.1')
     @db[:items].first.should == {:a=>Sequel::SQL::Blob.new(1.1.to_s)}
   end
-end if SQLITE_DB.adapter_scheme == :sqlite
+end if DB.adapter_scheme == :sqlite
 
 describe "An SQLite dataset" do
   before do
-    @d = SQLITE_DB[:items]
+    @d = DB[:items]
   end
   
   specify "should raise errors if given a regexp pattern match" do
-    proc{@d.literal(:x.like(/a/))}.should raise_error(Sequel::Error)
-    proc{@d.literal(~:x.like(/a/))}.should raise_error(Sequel::Error)
-    proc{@d.literal(:x.like(/a/i))}.should raise_error(Sequel::Error)
-    proc{@d.literal(~:x.like(/a/i))}.should raise_error(Sequel::Error)
-  end
-end
-
-describe "An SQLite numeric column" do
-  specify "should handle and return BigDecimal values" do
-    SQLITE_DB.create_table!(:d){numeric :d}
-    d = SQLITE_DB[:d]
-    d.insert(:d=>BigDecimal.new('80.0'))
-    d.insert(:d=>BigDecimal.new('NaN'))
-    d.insert(:d=>BigDecimal.new('Infinity'))
-    d.insert(:d=>BigDecimal.new('-Infinity'))
-    ds = d.all
-    ds.shift.should == {:d=>BigDecimal.new('80.0')}
-    ds.map{|x| x[:d].to_s}.should == %w'NaN Infinity -Infinity'
+    proc{@d.literal(Sequel.expr(:x).like(/a/))}.should raise_error(Sequel::Error)
+    proc{@d.literal(~Sequel.expr(:x).like(/a/))}.should raise_error(Sequel::Error)
+    proc{@d.literal(Sequel.expr(:x).like(/a/i))}.should raise_error(Sequel::Error)
+    proc{@d.literal(~Sequel.expr(:x).like(/a/i))}.should raise_error(Sequel::Error)
   end
 end
 
 describe "An SQLite dataset AS clause" do
   specify "should use a string literal for :col___alias" do
-    SQLITE_DB.literal(:c___a).should == "`c` AS 'a'"
+    DB.literal(:c___a).should == "`c` AS 'a'"
   end
 
   specify "should use a string literal for :table__col___alias" do
-    SQLITE_DB.literal(:t__c___a).should == "`t`.`c` AS 'a'"
+    DB.literal(:t__c___a).should == "`t`.`c` AS 'a'"
   end
 
   specify "should use a string literal for :column.as(:alias)" do
-    SQLITE_DB.literal(:c.as(:a)).should == "`c` AS 'a'"
+    DB.literal(Sequel.as(:c, :a)).should == "`c` AS 'a'"
   end
 
   specify "should use a string literal in the SELECT clause" do
-    SQLITE_DB[:t].select(:c___a).sql.should == "SELECT `c` AS 'a' FROM `t`"
+    DB[:t].select(:c___a).sql.should == "SELECT `c` AS 'a' FROM `t`"
   end
 
   specify "should use a string literal in the FROM clause" do
-    SQLITE_DB[:t___a].sql.should == "SELECT * FROM `t` AS 'a'"
+    DB[:t___a].sql.should == "SELECT * FROM `t` AS 'a'"
   end
 
   specify "should use a string literal in the JOIN clause" do
-    SQLITE_DB[:t].join_table(:natural, :j, nil, :a).sql.should == "SELECT * FROM `t` NATURAL JOIN `j` AS 'a'"
+    DB[:t].join_table(:natural, :j, nil, :table_alias=>:a).sql.should == "SELECT * FROM `t` NATURAL JOIN `j` AS 'a'"
   end
 end
 
 describe "SQLite::Dataset#delete" do
   before do
-    SQLITE_DB.create_table! :items do
+    DB.create_table! :items do
       primary_key :id
       String :name
       Float :value
     end
-    @d = SQLITE_DB[:items]
+    @d = DB[:items]
     @d.delete # remove all records
     @d << {:name => 'abc', :value => 1.23}
     @d << {:name => 'def', :value => 4.56}
@@ -321,10 +329,10 @@ describe "SQLite::Dataset#delete" do
   
   specify "should return the number of records affected when filtered" do
     @d.count.should == 3
-    @d.filter(:value.sql_number < 3).delete.should == 1
+    @d.filter{value < 3}.delete.should == 1
     @d.count.should == 2
 
-    @d.filter(:value.sql_number < 3).delete.should == 0
+    @d.filter{value < 3}.delete.should == 0
     @d.count.should == 2
   end
   
@@ -339,12 +347,12 @@ end
 
 describe "SQLite::Dataset#update" do
   before do
-    SQLITE_DB.create_table! :items do
+    DB.create_table! :items do
       primary_key :id
       String :name
       Float :value
     end
-    @d = SQLITE_DB[:items]
+    @d = DB[:items]
     @d.delete # remove all records
     @d << {:name => 'abc', :value => 1.23}
     @d << {:name => 'def', :value => 4.56}
@@ -362,38 +370,38 @@ end
 
 describe "SQLite dataset" do
   before do
-    SQLITE_DB.create_table! :test do
+    DB.create_table! :test do
       primary_key :id
       String :name
       Float :value
     end
-    SQLITE_DB.create_table! :items do
+    DB.create_table! :items do
       primary_key :id
       String :name
       Float :value
     end
-    @d = SQLITE_DB[:items]
+    @d = DB[:items]
     @d << {:name => 'abc', :value => 1.23}
     @d << {:name => 'def', :value => 4.56}
     @d << {:name => 'ghi', :value => 7.89}
   end
   after do
-    SQLITE_DB.drop_table?(:test, :items)
+    DB.drop_table?(:test, :items)
   end
   
   specify "should be able to insert from a subquery" do
-    SQLITE_DB[:test] << @d
-    SQLITE_DB[:test].count.should == 3
-    SQLITE_DB[:test].select(:name, :value).order(:value).to_a.should == \
+    DB[:test] << @d
+    DB[:test].count.should == 3
+    DB[:test].select(:name, :value).order(:value).to_a.should == \
       @d.select(:name, :value).order(:value).to_a
   end
     
   specify "should support #explain" do
-    SQLITE_DB[:test].explain.should be_a_kind_of(String)
+    DB[:test].explain.should be_a_kind_of(String)
   end
   
   specify "should have #explain work when identifier_output_method is modified" do
-    ds = SQLITE_DB[:test]
+    ds = DB[:test]
     ds.identifier_output_method = :upcase
     ds.explain.should be_a_kind_of(String)
   end
@@ -401,12 +409,15 @@ end
 
 describe "A SQLite database" do
   before do
-    @db = SQLITE_DB
+    @db = DB
     @db.create_table! :test2 do
       text :name
       integer :value
     end
   end
+  after do
+    @db.drop_table?(:test2)
+  end
 
   specify "should support add_column operations" do
     @db.add_column :test2, :xyz, :text
@@ -459,7 +470,7 @@ describe "A SQLite database" do
     @db[:test3].select(:id).all.should == [{:id => 1}, {:id => 3}]
   end
 
-  if SQLITE_DB.foreign_keys
+  if DB.foreign_keys
     specify "should keep foreign keys when dropping a column" do
       @db.create_table! :test do
         primary_key :id
@@ -533,6 +544,7 @@ describe "A SQLite database" do
     @db.from(table_name).all.should == [{:"s s"=>1}]
     @db.rename_column table_name, :"s s", :"t t"
     @db.from(table_name).all.should == [{:"t t"=>1}]
+    @db.drop_table?(table_name)
   end
   
   specify "should choose a temporary table name that isn't already used when dropping or renaming columns" do
@@ -574,6 +586,7 @@ describe "A SQLite database" do
     @db[:test3_backup1].columns.should == [:k]
     @db[:test3_backup2].columns.should == [:l]
     @db.loggers.delete(l)
+    @db.drop_table?(:test3, :test3_backup0, :test3_backup1, :test3_backup2)
   end
   
   specify "should support add_index" do
@@ -594,4 +607,44 @@ describe "A SQLite database" do
     @db.drop_column :a, :b
     @db.indexes(:a).should == {:a_a_index=>{:unique=>false, :columns=>[:a]}}
   end
-end  
+
+  specify "should have support for various #transaction modes" do
+    sqls = []
+    @db.loggers << Class.new{%w'info error'.each{|m| define_method(m){|sql| sqls << sql}}}.new
+
+    @db.transaction(:mode => :immediate) do
+      sqls.last.should == "BEGIN IMMEDIATE TRANSACTION"
+    end
+    @db.transaction(:mode => :exclusive) do
+      sqls.last.should == "BEGIN EXCLUSIVE TRANSACTION"
+    end
+    @db.transaction(:mode => :deferred) do
+      sqls.last.should == "BEGIN DEFERRED TRANSACTION"
+    end
+    @db.transaction do
+      sqls.last.should == Sequel::Database::SQL_BEGIN
+    end
+
+    @db.transaction_mode.should == nil
+    @db.transaction_mode = :immediate
+    @db.transaction_mode.should == :immediate
+    @db.transaction do
+      sqls.last.should == "BEGIN IMMEDIATE TRANSACTION"
+    end
+    @db.transaction(:mode => :exclusive) do
+      sqls.last.should == "BEGIN EXCLUSIVE TRANSACTION"
+    end
+
+    proc {@db.transaction_mode = :invalid}.should raise_error(Sequel::Error)
+
+    @db.transaction_mode.should == :immediate
+    proc {@db.transaction(:mode => :invalid) {}}.should raise_error(Sequel::Error)
+  end
+
+  specify "should keep unique constraints when copying tables" do
+    @db.alter_table(:test2){add_unique_constraint :name}
+    @db.alter_table(:test2){drop_column :value}
+    @db[:test2].insert(:name=>'a')
+    proc{@db[:test2].insert(:name=>'a')}.should raise_error(Sequel::ConstraintViolation)
+  end
+end
