@@ -2,7 +2,7 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 describe "Sequel::Model basic support" do 
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:items, :engine=>:InnoDB) do
       primary_key :id
       String :name
@@ -11,7 +11,7 @@ describe "Sequel::Model basic support" do
     end
   end
   after do
-    @db.drop_table(:items)
+    @db.drop_table?(:items)
     Object.send(:remove_const, :Item)
   end
 
@@ -107,7 +107,7 @@ describe "Sequel::Model basic support" do
 
     i.rb = true
     i.destroy.should be_nil
-    i.exists?.should be_true
+    i.exists?.should == true
     i.hooks.should == [:ad, :adr]
 
     i.name = 'K'
@@ -118,14 +118,14 @@ describe "Sequel::Model basic support" do
 
     i.rb = false
     i.destroy.should_not be_nil
-    i.exists?.should be_false
+    i.exists?.should == false
     i.hooks.should == [:ad, :adc]
   end
 
   specify "#exists? should return whether the item is still in the database" do
     i = Item.create(:name=>'J')
     i.exists?.should == true
-    Item.delete
+    Item.dataset.delete
     i.exists?.should == false
   end
 
@@ -139,10 +139,10 @@ describe "Sequel::Model basic support" do
     i = Item.create(:name=>'J', :num=>1)
     Item.all.should == [Item.load(:id=>1, :name=>'J', :num=>1)]
     i.set(:name=>'K', :num=>2)
-    i.save(:name)
+    i.save(:columns=>:name)
     Item.all.should == [Item.load(:id=>1, :name=>'K', :num=>1)]
     i.set(:name=>'L')
-    i.save(:num)
+    i.save(:columns=>:num)
     Item.all.should == [Item.load(:id=>1, :name=>'K', :num=>2)]
   end
   
@@ -159,12 +159,12 @@ describe "Sequel::Model basic support" do
   end
 
   specify ".to_hash should return a hash keyed on primary key if no argument provided" do
-    i = Item.create(:name=>'J')
+    Item.create(:name=>'J')
     Item.to_hash.should == {1=>Item.load(:id=>1, :name=>'J')}
   end
   
   specify ".to_hash should return a hash keyed on argument if one argument provided" do
-    i = Item.create(:name=>'J')
+    Item.create(:name=>'J')
     Item.to_hash(:name).should == {'J'=>Item.load(:id=>1, :name=>'J')}
   end
   
@@ -201,9 +201,9 @@ end
 
 describe "Sequel::Model with no existing table" do 
   specify "should not raise an error when setting the dataset" do
-    db = INTEGRATION_DB
-    db.drop_table(:items) if db.table_exists?(:items)
-    proc{class ::Item < Sequel::Model(db); end}.should_not raise_error
+    db = DB
+    db.drop_table?(:items)
+    proc{class ::Item < Sequel::Model(db); end; Object.send(:remove_const, :Item)}.should_not raise_error
     proc{c = Class.new(Sequel::Model); c.set_dataset(db[:items])}.should_not raise_error
   end
 end
